/*
 * mkd2html:  parse a markdown input file and generate a web page.
 *
 * usage:  mkd2html [header-fields] / [signature fields] < markdown > html
 *
 * example:
 *
 *   mkd2html ' <STYLE type="text/css">@import url(pages.css); </STYLE>' /
 *            '<HR>' '<P ALIGN=RIGHT>--generated by mk2html</P>' < text > html
 *
 */
/*
 * Copyright (C) 2007 David L Parsons.
 * The redistribution terms are provided in the COPYRIGHT file that must
 * be distributed with this source code.
 */
#include <stdio.h>

#include "config.h"

#include "mkdio.h"
#include "cstring.h"

extern char version[];

void
fail(char *why)
{
    fprintf(stderr, "md2html: %s\n", why);
    exit(1);
}


void
main(argc, argv)
char **argv;
{
    char *h;
    int i;
    void *input;

    if ( (input = mkd_in(stdin, 0)) == 0 )
	fail("can't read input");

    if ( !mkd_compile(input, 0) )
	fail("couldn't compile input");


    h = mkd_doc_title(input);

    /* print a header */

    puts("<!doctype html public \"-//W3C//DTD HTML 4.0 Transitional //EN\">");
    puts("<HTML>");
    puts("<HEAD>");
    printf("  <meta name=\"GENERATOR\" content=\"md2html\" %s>\n", version);
    puts("  <meta http-equiv=\"Content-Type\"");
    puts("        content=\"text/html; charset-us-ascii\">");
    if ( h ) {
	printf("  <title>");
	mkd_text(h, strlen(h), stdout, 0);
	puts("</title>");
    }
    for (i=1; (i < argc) && (strcmp(argv[i], "/") != 0); i++)
	printf("  %s\n", argv[i]);
    puts("</HEAD>");
    puts("<BODY>");

    /* print the compiled body */

    mkd_generatehtml(input, stdout);
    mkd_cleanup(input);

    i++;

    while ( i < argc )
	puts(argv[i++]);

    puts("</BODY>");
    puts("</HTML>");
    exit(0);
}
